@@ -2,14 +2,12 @@
 package Net::Object::Peer;
 
 use 5.10.0;
-use strict;
-use warnings;
-
-our $VERSION = '0.03'; # TRIAL
+use strictures 2;
 
 use Carp;
 our @CARP_NOT = qw( Beam::Emitter );
 
+use List::Util qw[ first uniqstr ];
 use Scalar::Util qw[ refaddr weaken ];
 use Data::OptList qw[ mkopt ];
 use Safe::Isa;
@@ -17,14 +15,17 @@ use Ref::Util qw[ is_arrayref ];
 use Types::Standard ':all';
 
 use Moo::Role;
+
 use MooX::ProtectedAttributes;
 
+our $VERSION = '0.04'; # TRIAL
+
 use Net::Object::Peer::Event;
 use Net::Object::Peer::UnsubscribeEvent;
 use Net::Object::Peer::Listener;
 use Net::Object::Peer::Emitter;
+use Net::Object::Peer::RefAddr;
 use Net::Object::Peer::Subscriptions;
-use Net::Object::Peer::Subscription;
 
 use Sub::QuoteX::Utils qw[ quote_subs ];
 
@@ -43,7 +44,6 @@ use Sub::QuoteX::Utils qw[ quote_subs ];
 #pod =cut
 
 use constant UnsubscribeEvent => __PACKAGE__ . '::UnsubscribeEvent';
-use constant Subscription     => __PACKAGE__ . '::Subscription';
 use constant Emitter          => __PACKAGE__ . '::Emitter';
 
 
@@ -54,7 +54,6 @@ has _subscriptions => (
     init_args => undef,
     isa       => InstanceOf ['Net::Object::Peer::Subscriptions'],
     default   => sub { Net::Object::Peer::Subscriptions->new },
-    handles   => { subscriptions => 'list' },
 );
 
 protected_has _emitter => (
@@ -64,6 +63,37 @@ protected_has _emitter => (
     handles  => [qw< emit_args send_args >],
 );
 
+#pod =attr addr
+#pod
+#pod A L<Net::Object::Peer::RefAddr> object providing a unique identity for this emitter.
+#pod
+#pod =cut
+
+has addr => (
+	     is => 'rwp',
+	     isa => InstanceOf[ 'Net::Object::Peer::RefAddr' ],
+	     init_arg => undef,
+	     predicate => 1,
+);
+
+#pod =begin pod_coverage
+#pod
+#pod =head4 has_addr
+#pod
+#pod =head4 BUILD
+#pod
+#pod =end pod_coverage
+#pod
+#pod =cut
+
+sub BUILD {}
+
+before BUILD => sub {
+
+    my $self = shift;
+    $self->_set_addr( Net::Object::Peer::RefAddr->new( $self ) );
+
+};
 
 
 #pod =method build_sub
@@ -122,10 +152,11 @@ sub build_sub {
                 quote_subs( [ delete $arg{code}, %arg ] );
             }
 
-	    else {
+            else {
 
-		croak( __PACKAGE__ . "::build_sub: can't figure out what to do with \%arg" );
-	    }
+                croak( __PACKAGE__
+                      . "::build_sub: can't figure out what to do with \%arg" );
+            }
         }
 
         elsif ( 'CODE' eq ref $arg ) {
@@ -146,7 +177,9 @@ sub build_sub {
 #pod   $self->subscribe( $peer, @event_tuple [, @event_tuple, ...  ] );
 #pod
 #pod Subscribe to one or more events sent by C<$peer>, which must consume
-#pod the L<Net::Object::Peer> role.
+#pod the L<Net::Object::Peer> role.  If C<$peer> additionally consumes the
+#pod L<Net::Object::Peer::Ephemeral> role, a strong reference to C<$peer>
+#pod is stored. (See  L<Net::Object::Peer::Cookbook/Translation/Proxy Nodes>.)
 #pod
 #pod The event name and the action to be performed when the event is
 #pod emitted are specified by a tuple with the following forms:
@@ -220,18 +253,18 @@ sub subscribe {
 
         my ( $name, $sub ) = @$event;
 
-        $_->unsubscribe foreach $self->_subscriptions->delete(
+        $self->_subscriptions->remove(
             name => $name,
             peer => $peer,
         );
 
         $self->_subscriptions->add(
-            Subscription->new(
-                name => $name,
-                peer => $peer,
-                unsubscribe =>
-                  $peer->_emitter->subscribe( $name, $sub, peer => $self ),
-            ) );
+            name => $name,
+            peer => $peer,
+            addr => Net::Object::Peer::RefAddr->new( $peer ),
+            unsubscribe =>
+              $peer->_emitter->subscribe( $name, $sub, peer => $self, ),
+        );
 
     }
 
@@ -252,6 +285,10 @@ sub subscribe {
 #pod   # Unsubscribe from one or more events emitted by a peer
 #pod   $self->unsubscribe( $peer, $event_name [, $event_name [, ... ]);
 #pod
+#pod   # Unsubscribe from the peer and event specified by the passed
+#pod   # Net::Object::Peer::Event object
+#pod   $self->unsubscribe( $event_object );
+#pod
 #pod   # Unsubscribe from one or more events emitted by all peers
 #pod   $self->unsubscribe( $event_name [, $event_name [, ... ] ] )
 #pod
@@ -259,6 +296,15 @@ sub subscribe {
 #pod event with a L<Net::Object::Peer::UnsubscribeEvent> as a payload will
 #pod be sent to affected peers who have subscribed to the unsubscribed event(s).
 #pod
+#pod C<$peer> may be either a L<Net::Object::Peer> or a
+#pod L<Net::Object::Peer::RefAddr> object.
+#pod
+#pod Note that B<Net::Object::Peer::Event> objects which are passed to
+#pod event handlers may have a masqueraded C<emitter> attribute.  Attempting
+#pod to unsubscribe from that C<emitter> is unwise.  Instead, pass either
+#pod the event object or the C<addr> field in that object, which is guaranteed
+#pod to identify the actual emitter subscribed to.
+#pod
 #pod =cut
 
 sub unsubscribe {
@@ -268,7 +314,9 @@ sub unsubscribe {
     return $self->_unsubscribe_all
       unless @_;
 
-    if ( $_[0]->$_does( __PACKAGE__ ) ) {
+    if (   $_[0]->$_does( __PACKAGE__ )
+        || $_[0]->$_isa( 'Net::Object::Peer::RefAddr' ) )
+    {
 
         # $peer, $name, ...
         return $self->_unsubscribe_from_peer_events( @_ )
@@ -277,6 +325,11 @@ sub unsubscribe {
         # $peer
         return $self->_unsubscribe_from_peer( @_ );
     }
+    elsif ( $_[0]->$_isa( 'Net::Object::Peer::Event' ) ) {
+
+	$self->_unsubscribe_from_peer_events( $_[0]->addr, $_[0]->name );
+
+    }
 
     # $name, ...
     return $self->_unsubscribe_from_events( @_ );
@@ -289,8 +342,7 @@ sub _unsubscribe_all {
 
     # say $self->name, ":\tunsubscribing from all peers";
 
-    $_->unsubscribe foreach $self->subscriptions;
-    $self->_subscriptions->clear;
+    $self->_subscriptions->remove;
 
     # signal peers that unsubscribe has happened.
 
@@ -301,37 +353,47 @@ sub _unsubscribe_all {
     return;
 }
 
+sub _find_peer_spec {
+
+    my $peer = shift;
+
+    return (
+          $peer->$_does( __PACKAGE__ )                  ? ( peer => $peer )
+        : $peer->isa( 'Net::Object::Peer::RefAddr' ) ? ( addr => $peer )
+        : croak( "can't grok \$peer: $peer\n" ),
+    );
+
+}
+
 sub _unsubscribe_from_peer_events {
 
     my ( $self, $peer ) = ( shift, shift );
 
-    my @unsubscribed;
+    my %spec = _find_peer_spec( $peer );
 
-    for my $name ( @_ ) {
+    my @unsubs = map {
+        my $name = $_;
+        map +{ name => $name, subs => $_ },
+          $self->_subscriptions->remove( %spec, name => $name )
+    } @_;
 
-        for my $subscription (
-            $self->_subscriptions->delete(
-                peer => $peer,
-                name => $name,
-            ) )
-        {
-            # say $self->name, ":\tunsubscribing from ", $peer->name, ":$name";
+    # if passed a refaddr, extract peer object from deleted subscriptions
+    if ( defined $spec{addr} ) {
 
-            $subscription->unsubscribe;
-            push @unsubscribed, $name;
-        }
+        # check for subs where peer is still alive
+        my $sub = first { defined $_->{peer} } map { $_->{subs} } @unsubs;
+
+        return unless defined $sub;
+
+        $peer = $sub->{peer};
     }
 
-    if ( @unsubscribed ) {
-
-        # say $self->name, ":\tnotifying ", $peer->name,
-        #   " of unsubscription from ", join( ', ', @unsubscribed );
+    if ( @unsubs ) {
 
         $self->send(
             $peer, 'unsubscribe',
             class       => UnsubscribeEvent,
-            event_names => \@unsubscribed,
-        );
+            event_names => [ uniqstr map { $_->{name} } @unsubs ] );
     }
 
     return;
@@ -344,10 +406,20 @@ sub _unsubscribe_from_peer {
 
     # say $self->name, ":\tunsubscribing from ", $peer->name;
 
-    $_->unsubscribe foreach $self->_subscriptions->delete( peer => $peer );
+    my %spec = _find_peer_spec( $peer );
 
-    # say $self->name, ":\tnotifying ", $peer->name,
-    #   " of unsubscription from all events";
+    my @unsubs = $self->_subscriptions->remove( %spec );
+
+    # if passed a refaddr, extract peer object from deleted subscriptions
+    if ( defined $spec{addr} ) {
+
+        # check for subs where peer is still alive
+        my $sub = first { defined $_->{peer} } @unsubs;
+
+        return unless defined $sub;
+
+        $peer = $sub->{peer};
+    }
 
     $self->send( $peer, 'unsubscribe', class => UnsubscribeEvent );
 
@@ -361,25 +433,27 @@ sub _unsubscribe_from_events {
     return unless @names;
 
     my %subs;
-    my @subs = $self->_subscriptions->delete(
+
+    my @subs = $self->_subscriptions->remove(
         sub {
             grep { $_[0]->name eq $_ } @names;
         } );
 
-    for my $sub ( @subs ) {
-        $sub->unsubscribe;
+    for my $sub ( grep { defined $_->{peer} } @subs ) {
 
-        my $list = $subs{ refaddr $sub->peer } ||= [ $sub->peer ];
-        push @$list, $sub->name;
+        my $list = $subs{ 0 + $sub->{addr} } ||= [ $sub->{peer} ];
+        push @$list, $sub->{name};
     }
 
     for my $sub ( values %subs ) {
 
-        my ( $peer, @names ) = @_;
-        $self->emit(
+        my ( $peer, @event_names ) = @$sub;
+
+        $self->send(
+            $peer,
             'unsubscribe',
             class       => UnsubscribeEvent,
-            event_names => \@names,
+            event_names => \@event_names,
         );
     }
 
@@ -387,13 +461,61 @@ sub _unsubscribe_from_events {
     return;
 }
 
+#pod =method detach
+#pod
+#pod   $self->detach;
+#pod
+#pod Detach the object from the network.  It will
+#pod
+#pod =over
+#pod
+#pod =item 1
+#pod
+#pod Unsubscribe from all events from all peers.
+#pod
+#pod =item 2
+#pod
+#pod Emit an C<unsubscribe> event with a L<Net::Object::Peer::UnsubscribeEvent> as a payload.
+#pod
+#pod =item 3
+#pod
+#pod Emit a C<detach> event.
+#pod
+#pod =back
+#pod
+#pod =cut
+
+sub detach {
+    my $self = shift;
+
+    $self->unsubscribe;
+    $self->emit( 'detach' );
+}
+
+
 #pod =method subscriptions
 #pod
+#pod   # return all subscriptions
 #pod   my @subscriptions = $self->subscriptions;
 #pod
-#pod Returns the events to which C<$self> is subscribed as a list of
-#pod L<Net::Object::Peer::Subscription> objects.
+#pod   # return matching subscriptions
+#pod   my @subscriptions = $self->subscriptions( $coderef | %spec );
 #pod
+#pod Returns the events to which C<$self> is subscribed as a list of
+#pod hashrefs (see L<Net::Object::Peer::Subscription::as_hashref>).  If
+#pod arguments are specified, only those which match are returned; see
+#pod L<Net::Object::Peer::Subscrition/find>;
+#pod
+#pod
+#pod =cut
+
+sub subscriptions {
+
+    my $self = shift;
+
+    return @_ ? $self->_subscriptions->find( @_ ) : $self->_subscriptions->list;
+}
+
 #pod =method emit
 #pod
 #pod   $self->emit( $event_name, %args );
@@ -401,7 +523,7 @@ sub _unsubscribe_from_events {
 #pod Broadcast the named event to all subscribed peers.  C<%args> contains
 #pod arguments to be passed the the payload class constructor.  The default
 #pod payload class is a L<Net::Object::Peer::Event> object; use the C<class> key to
-#pod specify an alternate class, which must be erived from B<Net::Object::Peer::Event>.
+#pod specify an alternate class, which must be derived from B<Net::Object::Peer::Event>.
 #pod
 #pod =cut
 
@@ -413,7 +535,7 @@ sub emit {
         $name,
         class   => 'Net::Object::Peer::Event',
         emitter => $self,
-        @_
+        @_,
     );
 }
 
@@ -435,7 +557,7 @@ sub send {
         $name,
         class   => 'Net::Object::Peer::Event',
         emitter => $self,
-        @_
+        @_,
     );
 }
 
@@ -476,7 +598,7 @@ sub __cb_unsubscribe {
 #pod
 #pod =cut
 
-sub DEMOLISH {}
+sub DEMOLISH { }
 
 around DEMOLISH => sub {
 
@@ -484,9 +606,8 @@ around DEMOLISH => sub {
 
     my ( $self, $in_global_destruction ) = @_;
 
-    # unsubscribe from network
-    $self->unsubscribe
-	unless $in_global_destruction;
+    $self->detach
+      unless $in_global_destruction;
 
     &$orig;
 };
@@ -512,7 +633,7 @@ Net::Object::Peer - Peer-to-Peer Publish/Subscribe Network of Objects
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 SYNOPSIS
 
@@ -640,6 +761,9 @@ C<_notify_subscribed> method will be invoked (if it exists) as
 
   $emitter->_notify_subscribed( $subscriber, @event_names );
 
+If the subscription already exists, it will be unsubscribed and
+then resubscribed.
+
 After a subscriber de-registers a handler, either explicitly via
 L</unsubscribe> or when the object is destroyed, it will L</emit> an
 C<unsubscribe> event with a L<Net::Object::Peer::UnsubscribeEvent>
@@ -649,6 +773,17 @@ While emitters are not automatically subscribed to C<"unsubscribe">
 events, this is easily accomplished by adding code to the emitters'
 C<_notify_subscribed> method.
 
+=head3 Detach Events
+
+When an object is destroyed, it emits a C<detach> event after
+unsubscribing from other peers' events.
+
+=head1 ATTRIBUTES
+
+=head2 addr
+
+A L<Net::Object::Peer::RefAddr> object providing a unique identity for this emitter.
+
 =head1 METHODS
 
 =head2 build_sub
@@ -682,7 +817,9 @@ handler.
   $self->subscribe( $peer, @event_tuple [, @event_tuple, ...  ] );
 
 Subscribe to one or more events sent by C<$peer>, which must consume
-the L<Net::Object::Peer> role.
+the L<Net::Object::Peer> role.  If C<$peer> additionally consumes the
+L<Net::Object::Peer::Ephemeral> role, a strong reference to C<$peer>
+is stored. (See  L<Net::Object::Peer::Cookbook/Translation/Proxy Nodes>.)
 
 The event name and the action to be performed when the event is
 emitted are specified by a tuple with the following forms:
@@ -727,6 +864,10 @@ for each subscription.
   # Unsubscribe from one or more events emitted by a peer
   $self->unsubscribe( $peer, $event_name [, $event_name [, ... ]);
 
+  # Unsubscribe from the peer and event specified by the passed
+  # Net::Object::Peer::Event object
+  $self->unsubscribe( $event_object );
+
   # Unsubscribe from one or more events emitted by all peers
   $self->unsubscribe( $event_name [, $event_name [, ... ] ] )
 
@@ -734,12 +875,49 @@ Unsubscribe from events/peers. After unsubscription, an I<unsubscribe>
 event with a L<Net::Object::Peer::UnsubscribeEvent> as a payload will
 be sent to affected peers who have subscribed to the unsubscribed event(s).
 
+C<$peer> may be either a L<Net::Object::Peer> or a
+L<Net::Object::Peer::RefAddr> object.
+
+Note that B<Net::Object::Peer::Event> objects which are passed to
+event handlers may have a masqueraded C<emitter> attribute.  Attempting
+to unsubscribe from that C<emitter> is unwise.  Instead, pass either
+the event object or the C<addr> field in that object, which is guaranteed
+to identify the actual emitter subscribed to.
+
+=head2 detach
+
+  $self->detach;
+
+Detach the object from the network.  It will
+
+=over
+
+=item 1
+
+Unsubscribe from all events from all peers.
+
+=item 2
+
+Emit an C<unsubscribe> event with a L<Net::Object::Peer::UnsubscribeEvent> as a payload.
+
+=item 3
+
+Emit a C<detach> event.
+
+=back
+
 =head2 subscriptions
 
+  # return all subscriptions
   my @subscriptions = $self->subscriptions;
 
+  # return matching subscriptions
+  my @subscriptions = $self->subscriptions( $coderef | %spec );
+
 Returns the events to which C<$self> is subscribed as a list of
-L<Net::Object::Peer::Subscription> objects.
+hashrefs (see L<Net::Object::Peer::Subscription::as_hashref>).  If
+arguments are specified, only those which match are returned; see
+L<Net::Object::Peer::Subscrition/find>;
 
 =head2 emit
 
@@ -748,7 +926,7 @@ L<Net::Object::Peer::Subscription> objects.
 Broadcast the named event to all subscribed peers.  C<%args> contains
 arguments to be passed the the payload class constructor.  The default
 payload class is a L<Net::Object::Peer::Event> object; use the C<class> key to
-specify an alternate class, which must be erived from B<Net::Object::Peer::Event>.
+specify an alternate class, which must be derived from B<Net::Object::Peer::Event>.
 
 =head2 send
 
@@ -785,6 +963,14 @@ specified peer.
 
 =begin pod_coverage
 
+=head4 has_addr
+
+=head4 BUILD
+
+=end pod_coverage
+
+=begin pod_coverage
+
 =head3 DEMOLISH
 
 =end pod_coverage
@@ -877,6 +1063,9 @@ __END__
 #pod
 #pod   $emitter->_notify_subscribed( $subscriber, @event_names );
 #pod
+#pod If the subscription already exists, it will be unsubscribed and
+#pod then resubscribed.
+#pod
 #pod After a subscriber de-registers a handler, either explicitly via
 #pod L</unsubscribe> or when the object is destroyed, it will L</emit> an
 #pod C<unsubscribe> event with a L<Net::Object::Peer::UnsubscribeEvent>
@@ -886,4 +1075,7 @@ __END__
 #pod events, this is easily accomplished by adding code to the emitters'
 #pod C<_notify_subscribed> method.
 #pod
+#pod =head3 Detach Events
 #pod
+#pod When an object is destroyed, it emits a C<detach> event after
+#pod unsubscribing from other peers' events.
