@@ -2,12 +2,13 @@
 package Net::Object::Peer::Emitter;
 
 use 5.10.0;
-use strict;
-use warnings;
-
-our $VERSION = '0.03'; # TRIAL
 
 use Moo;
+use strictures 2;
+use namespace::clean;
+
+our $VERSION = '0.04'; # TRIAL
+
 with 'BeamX::Peer::Emitter';
 
 around subscribe => sub {
@@ -39,7 +40,7 @@ Net::Object::Peer::Emitter - Peer-to-Peer Publish/Subscribe Network of Objects
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 DESCRIPTION
 
