@@ -3,16 +3,15 @@
 package Net::Object::Peer::UnsubscribeEvent;
 
 use 5.10.0;
-use strict;
-use warnings;
-
-our $VERSION = '0.03'; # TRIAL
 
 use Types::Standard qw[ ArrayRef Str is_ArrayRef ];
 
+use Moo;
+use strictures 2;
 use namespace::clean;
 
-use Moo;
+our $VERSION = '0.04'; # TRIAL
+
 extends 'Net::Object::Peer::Event';
 
 #pod =attr event_names
@@ -46,7 +45,7 @@ Net::Object::Peer::UnsubscribeEvent - The payload for a Net::Object::Peer unsubs
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 DESCRIPTION
 
